#!/bin/ksh
#-------------------------------------------------
# Big Brother - EMC CLARiiON FC-4500 Hardware Agent
#
#	This agent monitor the hardware of an EMC 
#  CLARiiON FC-4500 Array using the Navisphere
#	Command Line Interface (NAVICLI)
#
#	original Author:   Sabey (sabey2000@hotmail.com)
#	adapted to FC4500: Uwe Kirbach (u.kirbach@enbw.com)
#
#	Feel free to improve or modify.
#
#-------------------------------------------------
#
###################################################################
#
# Requirements:
#  
# For each host that is an Clariion FibreCat you want to monitor modify the
# bb-hosts file as appropriate, the correct syntax is 
# IP address	HostName	# clariion
# i.e.
# 192.186.200.40 Host1 # clariion
#
# Modify $BBHOME/etc/bb-bbexttab to include this script in the BBEXT variable
# of one central machine.
# Stop and restart BB.
#
###################################################################

BBPROG=CLARiiON; export BBPROG

#
TEST="hw"
CHECK_TEST="clariion4500"

NAVICLI=/opt/Navisphere/bin/navicli	# Where is the NAVICLI cmd
GAWK=/usr/local/bin/awk				# need GNU AWK

# for testing purposes only
#
#BBHOME="/usr/local/perf/bb"
#FIBRECAT_NAME="S2A6020"

##---[Check BBHOME is set]--------------------------------
[[ "$BBHOME" = "" ]] &&
{
   echo "BBHOME is not set... exiting"; exit 1
}

[[ ! -x $GAWK ]] && 
{
	echo "This agent need GNU Awk, check GAWK variable"; exit 1
}

#---[Load bb global parameter]---------------------------
[[ "$BBTMP" == "" ]] && . $BBHOME/etc/bbdef.sh

COLOR=green
MYTMPFILE=$BBTMP/${CHECK_TEST}_TMP.$$			# Use a buffer tmp file for NAVICLI output
MYOUTFILE=$BBTMP/${CHECK_TEST}_OUT.$$			# Use a buffer tmp file for NAVICLI output
YELLOWLINE_FILE=$BBTMP/${CHECK_TEST}_Yellow.$$
REDLINE_FILE=$BBTMP/${CHECK_TEST}_Red.$$

$TOUCH $YELLOWLINE_FILE $REDLINE_FILE

get_status()
{
#set -x

	# FC4500
	# Checks only via direct FC communication
	# because the Navisphere Agent runs not on the FibreCat.
	# It runs on the host, which is connected to the FibreCat 4500
	# Here: Look for FC devices for inband-communication with the FC4500

	DEVICE=$1

   #---[1. Get cache status]---------------------------------------------------------
   $SUDO $NAVICLI -d $DEVICE getcache > $MYTMPFILE

   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		YELLOWLINE=""
		REDLINE=""
		print "<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>SP Cache</b></font></th></tr>"
		print "<tr><td><table width=\"100%\" border=0><tbody><tr>"
	}
	{
		if ($0 ~ /SP Write|Read Cache State/) {
			split($0, status, " State    *")
			icon=""
			if (status[2] ~ /Enabled/) {
				icon="&green"
			} else {
				icon="&yellow"
			}
			if ($0 !~ /SP Read Cache/) {
				printf("<td align=center>%s %s (%s)</td>\n", icon, status[1], status[2])
				if (icon == "&yellow") 
				{	YELLOWLINE=YELLOWLINE sprintf("%s %s (%s)\n", icon, status[1], status[2]) }
			}
		}
	}
	END {
		if (REDLINE != "") { printf("%s\n", REDLINE) >> REDFILE }
		if (YELLOWLINE != "") { printf("%s\n", YELLOWLINE) >> YELLOWFILE }
		print "</tr></tbody></table></td></tr></tbody></table>"
	}' >> $MYOUTFILE

   #---[2. Port status]--------------------------------------------------------------

  	$SUDO $NAVICLI -d $DEVICE port -list > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		REDLINE=""
		YELLOWLINE=""
		print "<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>Port Status</b></font></th></tr>"
		print "<tr><td><table width=\"100%\" border=0><tbody><tr>"
	}
   {
      if ($0 ~ /Information about each SPPORT:/) {
         FLAG=1
      }
      if (FLAG == 1) {
         if (($0 ~ /SP Name/) || ($0 ~ /SP Port ID/) || ($0 ~ /Link Status/) || ($0 ~ /Port Status/)) {
				split($0, status, ":    *")
            if (status[1] ~ /SP Name/) {
					SP=status[2]
            }
            if (status[1] ~ /Link Status/) {
					LinkS=status[2]
               if (LinkS ~ /Up/) {
						LinkIcon="&green"
					} else {
						LinkIcon="&red"
					}
            }
            if (status[1] ~ /Port Status/) {
					PortS=status[2]
               if (PortS ~ /Online/) {
						PortIcon="&green"
					} else {
						PortIcon="&red"
					}
					printf("<td align=center>%s %s Port (%s) %s Link (%s)</td>\n", SP, PortIcon, PortS, LinkIcon, LinkS)
					if (LinkIcon ~ /red/ || PortIcon ~ /red/) {
						REDLINE=REDLINE sprintf("&red %s %s Port (%s) and %s Link (%s)\n", SP, PortIcon, PortS, LinkIcon, LinkS)
					}
            }
         }
      }
	}
	END {
		if (REDLINE != "") { printf("%s\n", REDLINE) >> REDFILE }
		print "</tr></tbody></table></td></tr></tbody></table>"
	}' >> $MYOUTFILE

   #---[3. DPE/DAE Status]-----------------------------------------------------------
  	$SUDO $NAVICLI -d $DEVICE getcrus > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		FLAG=0
		REDLINE=""
		YELLOWLINE=""
		sps_failed=0
	}
   {
      if (($0 ~ /[DPE|DAE] Enclosure ./) || ($0 ~ /State/)) {
         if ($0 ~ /State/) {
				enclosure=$0
				sub(" [A-Za-z]+ . State: * .*","",enclosure)
				sub("Enclosure .", "")
				split($0, status, " State:  *")
				module_name=status[1]
				module_state=status[2]
            if (module_state ~ /Present/) {
					module_state="Ok"
					icon="&green"
            } else if (module_state ~ /Empty/) {
					module_state="<b>"module_state"</b>"
					module_name="<b>"module_name"</b>"
            	if (module_name ~ /SPS/) {
						sps_failed++
						sps_fail[module_name]=module_state
					} else if (module_name ~ /Power/) {
						power_failed[enclosure]++
						power_fail[enclosure module_name]=module_state
					}
					icon="&clear"
            } else if (module_state ~ /[C|c]harg/ && module_name ~ /SPS/) {
					icon="&green"
					module_state="<b>"module_state"</b>"
					module_name="<b>"module_name"</b>"
            } else {
					module_state="<b>"module_state"</b>"
					module_name="<b>"module_name"</b>"
            	if (module_name ~ /SPS/) {
						sps_failed++
						sps_fail[module_name]=module_state
						icon="&yellow"
					} else if (module_name ~ /Power/) {
						power_failed[enclosure]++
						power_fail[enclosure module_name]=module_state
						icon="&yellow"
					} else {
						icon="&red"
						REDLINE=REDLINE sprintf("%s %s %s (%s) failed\n", icon, enclosure, module_name, module_state)
					}
            }
				printf("<td align=center>%s %s (%s)</td>\n", icon, module_name, module_state)
         }
         if ($0 ~ /[DPE|DAE] Enclosure ./) {
				if (FLAG == 1) {
					print "</tr></tbody></table></td></tr></tbody></table>"
				} else {
					FLAG=1
				}
				printf("<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>%s</b></font></th></tr>", $0)
				print "<tr><td><table width=\"100%\" border=0><tbody><tr>"
         }
      }
	}
	END {
		if (sps_failed == 2) {
			icon="&red"
			REDLINE=REDLINE sprintf("%s No Standby Power Supply available\n", icon)
			for (sps in sps_fail) {
				REDLINE=REDLINE sprintf("   %s %s (%s)\n", icon, sps, sps_fail[sps])
			}
		} else if (sps_failed == 1) {
			icon="&yellow"
			for (sps in sps_fail) {
				if (sps_fail[sps] != "Empty") {
					YELLOWLINE=YELLOWLINE sprintf("%s %s (%s) failed\n", icon, sps, sps_fail[sps])
				}
			}
		}
		for (encl in power_failed) {
			if (power_failed[encl] == 2) {
				icon="&red"
				REDLINE=REDLINE sprintf("%s No Power Supply in %s available\n", icon, encl)
				for (power in power_fail) {
					if (power ~ encl) {
						REDLINE=REDLINE sprintf("   %s %s (%s)\n", icon, power, power_fail[power])
					}
				}
			} else if (power_failed[encl] == 1) {
				icon="&yellow"
				for (power in power_fail) {
					if (power_fail[power] != "Empty") {
						if (power ~ encl) {
							YELLOWLINE=YELLOWLINE sprintf("%s %s (%s) failed\n", icon, power, power_fail[power])
						}
					}
				}
			}
		}
		if (REDLINE != "") { printf("%s\n", REDLINE) >> REDFILE }
		if (YELLOWLINE != "") { printf("%s\n", YELLOWLINE) >> YELLOWFILE }
		print "</tr></tbody></table></td></tr></tbody></table>"
	}' >> $MYOUTFILE

   #---[4. Disks status]-------------------------------------------------------------

  	$SUDO $NAVICLI -d $DEVICE getdisk -state > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		FLAG=0
		hot_spares=0
		REDLINE=""
		YELLOWLINE=""
		print "<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>Disk Status</b></font></th></tr>"
		print "<table width=\"100%\" border=\"1\">\n<tbody><tr><th align=\"center\">Enclosure</th><th align=\"center\">Disk 0</th><th align=\"center\">Disk 1</th><th align=\"center\">Disk 2</th><th align=\"center\">Disk 3</th><th align=\"center\">Disk 4</th><th align=\"center\">Disk 5</th><th align=\"center\">Disk 6</th><th align=\"center\">Disk 7</th><th align=\"center\">Disk 8</th><th align=\"center\">Disk 9</th></tr>"
	}
	{
		if (FLAG == 1) {
			sub ("   *","")
			sub ("State:","")
			icon=""
			status=$0
	
			FLAG=0
			if (status ~ /Hot Spare/) {
				sub ("Hot Spare ","",status)
				status="Hot Spare"
				hot_spares++
			}
			if (status ~ /Empty/) { icon="&clear" }
			else {
				if ((status ~ /Enabled/) || (status ~ /Ready/) || (status ~ /Hot Spare/) || (status ~ /Unbound/) || (status ~ /Binding/) || (status ~ /Expanding/)) { icon="&green" }
				else {
					icon="&yellow"
					YELLOWLINE=YELLOWLINE sprintf("%s Disk %s_%s failed\n", icon, disk[1], disk[2])
				}
			}
			if ((status ~ /Ready/) || (status ~ /Unbound/) || (status ~ /Binding/) || (status ~ /Expanding/) || (status ~ /Removed/))
			{ status="<b>"status"</b>" }
			if (disk[2] == 0)
			{
				printf("<tr><td align=\"left\">Encl. %s</td><td align=\"left\">%s %s</td>\n", disk[1], icon, status)
			}
			else
			{ 
				if (disk[2] == 9) {
					printf("<td align=\"left\">%s %s</td></tr>\n", icon, status)
				} else {
					printf("<td align=\"left\">%s %s</td>\n", icon, status) 
				}
			}
		}
		if (($0 ~ /Enclosure . Disk ./) && (FLAG == 0)) {
			sub ("Enclosure ","")
			sub (" Disk ","-")
			split($0,disk,"-")
			FLAG=1
		}
	}
	END {
		if (REDLINE != "" || YELLOWLINE != "") {
			if (hot_spares == 0) {
				printf("&red Disks are failed and no additional hot spares available\n") >> REDFILE
				printf("%s\n", YELLOWLINE) >> REDFILE
			} else {
				printf("&yellow Disks are failed and only %s hot spares available\n", hot_spares) >> YELLOWFILE
				printf("%s\n", YELLOWLINE) >> YELLOWFILE
			}
		}
		print "</tr></tbody></table></td></tr></tbody></table>"
		
	}' >> $MYOUTFILE

   #---[Generate information to send]---
   LINE="$($CAT $REDLINE_FILE | $SED "s/^[ 	]*$//;s/\&yellow/\&red/g")
$($CAT $YELLOWLINE_FILE | $SED "s/^[ 	]*$//")"

   #---[Check global color output]---
   YELLOWS=$(echo $LINE | $GREP \&yellow)
   [[ "$YELLOWS" != "" ]] && COLOR=yellow

   REDS=$(echo $LINE | $GREP \&red)
   if [ "$REDS" != "" ]; then
		# Check, if the SPS are recharging
		SPS_TROUBLE=$(echo $REDS | $GREP "red SPS ")
		if [ "$SPS_TROUBLE" != "" ]; then
			SPS_TESTDAY=$($SUDO $NAVICLI -d $DEVICE getcache -bbutd | $AWK '{print $4}')
			SP_CURRENTDAY=$($SUDO $NAVICLI -d $DEVICE getcontrol -day | $AWK '{print $5}')
			if [ "$SPS_TESTDAY" = "$SP_CURRENTDAY" ]; then
				SPS_TESTHOUR=$($SUDO $NAVICLI -d $DEVICE getcache -bbutt | $AWK '{print $4}' | $CUT -d":" -f1)
				SP_CURRENTHOUR=$($SUDO $NAVICLI -d $DEVICE getcontrol -time | $AWK '{print $3}' | $CUT -d":" -f1)
				(( TIME_DIFF = $SP_CURRENTHOUR - $SPS_TESTHOUR ))
				if [ $TIME_DIFF -le 1 ]; then
					COLOR=green
					SPS_CHARGING="&green SPS is recharging - red lights on \"SP Write Cache\" and \"SPS\" are normal in this situation"
				else
					COLOR=yellow
					SPS_CHARGING="&yellow SPS is recharging more than 1 hour - too long (since $SPS_TESTHOUR clock, now it's $SP_CURRENTHOUR)"
				fi
			else
				COLOR=yellow
				SPS_CHARGING="&yellow SPS is recharging on $SPS_TESTDAY - Today is $SP_CURRENTDAY"
			fi
		else
      	COLOR=red
			SPS_CHARGING=""
		fi
   else
		SPS_CHARGING=""
   fi

   #---[Send page info to BBDISPLAY]---
   LINE="status $SP_FULLNAME.$TEST $COLOR $($DATE)
$HEADER
$SPS_CHARGING
$LINE
$($CAT $MYOUTFILE)"

   $BB $BBDISP "$LINE"

   $RM -f $MYOUTFILE $MYTMPFILE $YELLOWLINE_FILE $REDLINE_FILE

set +x
}

#
# Test-Loop
#
test_loop()
{
#set -x

	$GREP "^[0-9]" $BBHOSTS | $GREP ${CHECK_TEST} |
	while read linein
	do
		SP_IP=""
		SP_FULLNAME=""
		set $linein
		if [ "$#" -ne 0 ]; then
			SP_IP=$1
			SP_FULLNAME=$2
			COLOR="green"
			if [ "$FIBRECAT_NAME" = "$SP_FULLNAME" ]; then
				LINE=""
				DEVICES=$($SUDO $NAVICLI -h $MACHINE getagent -node | $EGREP "^Node:" | $SED 's/Node: *//')
				for device in $DEVICES
				do
					SP_SERIAL=${device#*-}
					[[ "$SP_SERIAL" == "$FIBRECAT_SERIAL" ]] &&
					{
 						ARRAYNAME=$($SUDO $NAVICLI -d $device arrayname | $AWK '{print $3}')
						if [[ "$ARRAYNAME" != "" ]]; then
		   				#---[Collect DATA]---------------------------------------
			   			HEADER="<h3>Status of $CHECK_TEST Storage Array $ARRAYNAME</h3>"
							get_status $device
							break
						fi
					}
				done
			fi
		fi
	done
}

#
# Main
#
# now the FibreCat FC4500
test_loop

#######
# fini
#######
