#!/bin/ksh
#-------------------------------------------------
# Big Brother - EMC CLARiiON FC-4700 Hardware Agent
#
#	This agent monitor the hardware of an EMC 
#  CLARiiON FC-4700 Array using the Navisphere
#	Command Line Interface (NAVICLI)
#
#	original Author:   Sabey (sabey2000@hotmail.com)
#
#	Feel free to improve or modify.
#
#-------------------------------------------------
#
###################################################################
#
# Requirements:
#  
# For each host that is an Clariion FibreCat you want to monitor modify the
# bb-hosts file as appropriate, the correct syntax is 
# IP address	HostName	# clariion4700
# i.e.
# 192.186.200.40 Host1 # clariion4700
#
# If you have unused FC-Ports in your Clariion, you can define them
# 192.186.200.40 Host1 # clariion4700:UNUSED_FC_PORTS=SPA1,SPB1
# 192.186.200.41 Host2 # clariion4700:UNUSED_FC_PORTS=spa1,spb1
#
# Modify $BBHOME/etc/bb-bbexttab to include this script in the BBEXT variable
# of one central machine.
# Stop and restart BB.
#
###################################################################

BBPROG=$(basename $0); export BBPROG

#
TEST="hw"
CHECK_TEST="clariion4700"

set -A allowed_parameter "UNUSED_FC_PORTS"

spacer="   "

NAVICLI=/opt/Navisphere/bin/navicli		# Where is the navicli cmd
GAWK=/usr/local/bin/awk						# need GNU AWK

# for testing purposes only
#
#BBHOME="/usr/local/perf/bb"
#SP_FULLNAME="S2A6024"
#SP_IP="10.145.6.24"
#UNUSED_FC="SPA1,SPB1"
#BB=echo

##---[Check BBHOME is set]--------------------------------
[[ "$BBHOME" = "" ]] &&
{
	echo "BBHOME is not set... exiting"; exit 1
}

[[ ! -x $GAWK ]] &&
{
	echo "This agent needs GNU Awk, check GAWK variable"; exit 1
}

[[ ! -x $NAVICLI ]] &&
{
	echo "This agent needs EMC Navisphere navicli, check NAVICLI variable"; exit 1
}

#---[Load bb global parameter]---------------------------
[[ "$BBTMP" == "" ]] && . $BBHOME/etc/hobbitclient.cfg

COLOR=green
MYTMPFILE=$BBTMP/${CHECK_TEST}_TMP.$$			# Use a buffer tmp file for NAVICLI output
MYOUTFILE=$BBTMP/${CHECK_TEST}_OUT.$$			# Use a buffer tmp file for NAVICLI output

get_status()
{

	sp_ip=$1

	YELLOWLINE_FILE=$BBTMP/${CHECK_TEST}_Yellow.$$
	REDLINE_FILE=$BBTMP/${CHECK_TEST}_Red.$$
	$TOUCH $YELLOWLINE_FILE $REDLINE_FILE

	ARRAYNAME=$($SUDO $NAVICLI -h $SP_IP arrayname | $AWK '{print $3}')
	if [[ "$ARRAYNAME" != "" ]]; then
		#---[Collect DATA]---------------------------------------
		HEADER="<h3>Status of $CHECK_TEST Storage Array $ARRAYNAME</h3>"
	else
		# Array doesn't respond (Service processor)
		COLOR="red"

   	#---[Send page info to BBDISPLAY]---
	   LINE="status $SP_FULLNAME.$TEST $COLOR $($DATE)
${spacer}&red Service processor $sp_ip of $SP_FULLNAME does not respond"

	   $BB $BBDISP "$LINE"

		$RM -f $YELLOWLINE_FILE $REDLINE_FILE
		return
	fi


   #---[1. Get cache status]---------------------------------------------------------
   $SUDO $NAVICLI -h $sp_ip getcache > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		YELLOWLINE=""
		REDLINE=""
		spacer="   "
		print "<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>SP Cache</b></font></th></tr>"
		print "<tr><td><table width=\"100%\" border=0><tbody><tr>"
	}
	{
		if ($0 ~ /SP Write|Read Cache State/) {
			split($0, status, " State    *")
			icon=""
			if (status[2] ~ /Enabled/) {
				icon="&green"
			} else {
				icon="&yellow"
			}
			if ($0 !~ /SP Read Cache/) {
				printf("<td align=center>%s %s (%s)</td>\n", icon, status[1], status[2])
				if (icon == "&yellow") 
				{	YELLOWLINE=YELLOWLINE sprintf("%s%s %s (%s)\n", spacer, icon, status[1], status[2]) }
			}
		}
	}
	END {
		if (REDLINE != "") { printf("%s\n", REDLINE) >> REDFILE }
		if (YELLOWLINE != "") { printf("%s\n", YELLOWLINE) >> YELLOWFILE }
		print "</tr></tbody></table></td></tr></tbody></table>"
	}' >> $MYOUTFILE

   #---[2. Port status]--------------------------------------------------------------

   $SUDO $NAVICLI -h $sp_ip port -list > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v UNUSED_FC="$UNUSED_FC_PORTS" -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		UNUSED=split(UNUSED_FC, UFC, ",") 
		CHECK_SP=""
		REDLINE=""
		YELLOWLINE=""
		spacer="   "
		SPPort=""
		print "<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>Port Status</b></font></th></tr>"
		print "<tr><td><table width=\"100%\" border=1><tbody><tr>"
	}
   {
      if ($0 ~ /Information about each SPPORT:/) {
         FLAG=1
      }
      if (FLAG == 1) {
         if (($0 ~ /SP Name/) || ($0 ~ /SP Port ID/) || ($0 ~ /Link Status/) || ($0 ~ /Port Status/)) {
				split($0, status, ":    *")
            if (status[1] ~ /SP Name/) {
					SP=status[2]
            }
            if (status[1] ~ /SP Port ID/) {
					PortID=status[2]
					SPPort=SP PortID
					gsub(" ", "", SPPort)
            }
            if (status[1] ~ /Link Status/) {
					LinkS=status[2]
               if (LinkS ~ /Up/) {
						LinkIcon="&green"
					} else {
						if ( UNUSED > 0 ) {
							for (i in UFC) {
								if ( SPPort == UFC[i] ) {
									LinkIcon="&clear"
									break
								} else { LinkIcon="&red" }
							}
						} else { LinkIcon="&red" }
					}
            }
            if (status[1] ~ /Port Status/) {
					PortS=status[2]
               if (PortS ~ /Online/) {
						PortIcon="&green"
					} else {
						if ( UNUSED ) {
							for (i in UFC) {
								if ( SPPort == UFC[i] ) {
									PortIcon="&clear"
									break
								} else { PortIcon="&red" }
							}
						} else { PortIcon="&red" }
					}
					printf("<td align=center>%s %s Port (%s) %s Link (%s)</td>\n", SPPort, PortIcon, PortS, LinkIcon, LinkS)
					if (LinkIcon ~ /red/ || PortIcon ~ /red/) {
						REDLINE=REDLINE sprintf("%s&red %s %s Port (%s) and %s Link (%s)\n", spacer, SPPort, PortIcon, PortS, LinkIcon, LinkS)
					}
            }
         }
      }
	}
	END {
		if (REDLINE != "") { printf("%s\n", REDLINE) >> REDFILE }
		print "</tr></tbody></table></td></tr></tbody></table>"
	}' >> $MYOUTFILE

   #---[3. DPE/DAE Status]-----------------------------------------------------------
  	$SUDO $NAVICLI -h $sp_ip getcrus > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		FLAG=0
		REDLINE=""
		YELLOWLINE=""
		spacer="   "
		sps_failed=0
	}
   {
      if (($0 ~ /[DPE|DAE] Bus . Enclosure ./) || ($0 ~ /State/)) {
         if ($0 ~ /State/) {
				enclosure=$0
				sub(" [A-Za-z]+ . State: * .*","",enclosure)
				sub("Bus . Enclosure .", "")
				split($0, status, " State:  *")
				module_name=status[1]
				module_state=status[2]
            if (module_state ~ /Present/) {
					module_state="Ok"
					icon="&green"
            } else if (module_state ~ /Empty/) {
					module_state="<b>"module_state"</b>"
					module_name="<b>"module_name"</b>"
            	if (module_name ~ /SPS/) {
						sps_failed++
						sps_fail[module_name]=module_state
					} else if (module_name ~ /Power/) {
						power_failed[enclosure]++
						power_fail[enclosure module_name]=module_state
					}
					icon="&clear"
            } else if (module_state ~ /[C|c]harg/ && module_name ~ /SPS/) {
					icon="&green"
					module_state="<b>"module_state"</b>"
					module_name="<b>"module_name"</b>"
            } else {
					module_state="<b>"module_state"</b>"
					module_name="<b>"module_name"</b>"
            	if (module_name ~ /SPS/) {
						sps_failed++
						sps_fail[module_name]=module_state
						icon="&yellow"
					} else if (module_name ~ /Power/) {
						power_failed[enclosure]++
						power_fail[enclosure module_name]=module_state
						icon="&yellow"
					} else {
						icon="&red"
						REDLINE=REDLINE sprintf("%s%s %s %s (%s) failed\n", spacer, icon, enclosure, module_name, module_state)
					}
            }
				printf("<td align=center>%s %s (%s)</td>\n", icon, module_name, module_state)
         }
         if ($0 ~ /[DPE|DAE] Bus . Enclosure ./) {
				if (FLAG == 1) {
					print "</tr></tbody></table></td></tr></tbody></table>"
				} else {
					FLAG=1
				}
				printf("<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>%s</b></font></th></tr>", $0)
				print "<tr><td><table width=\"100%\" border=0><tbody><tr>"
         }
      }
	}
	END {
		if (sps_failed == 2) {
			icon="&red"
			REDLINE=REDLINE sprintf("%s%s No Standby Power Supply available\n", spacer, icon)
			for (sps in sps_fail) {
				REDLINE=REDLINE sprintf("%s%s%s %s (%s)\n", spacer, spacer, icon, sps, sps_fail[sps])
			}
		} else if (sps_failed == 1) {
			icon="&yellow"
			for (sps in sps_fail) {
				if (sps_fail[sps] != "Empty") {
					YELLOWLINE=YELLOWLINE sprintf("%s%s %s (%s) failed\n", spacer, icon, sps, sps_fail[sps])
				}
			}
		}
		for (encl in power_failed) {
			if (power_failed[encl] == 2) {
				icon="&red"
				REDLINE=REDLINE sprintf("%s%s No Power Supply in %s available\n", spacer, icon, encl)
				for (power in power_fail) {
					if (power ~ encl) {
						REDLINE=REDLINE sprintf("%s%s%s %s (%s)\n", spacer, spacer, icon, power, power_fail[power])
					}
				}
			} else if (power_failed[encl] == 1) {
				icon="&yellow"
				for (power in power_fail) {
					if (power_fail[power] != "Empty") {
						if (power ~ encl) {
							YELLOWLINE=YELLOWLINE sprintf("%s%s %s (%s) failed\n", spacer, icon, power, power_fail[power])
						}
					}
				}
			}
		}
		if (REDLINE != "") { printf("%s\n", REDLINE) >> REDFILE }
		if (YELLOWLINE != "") { printf("%s\n", YELLOWLINE) >> YELLOWFILE }
		print "</tr></tbody></table></td></tr></tbody></table>"
	}' >> $MYOUTFILE

   #---[4. Disks status]-------------------------------------------------------------

  	$SUDO $NAVICLI -h $sp_ip getdisk -state > $MYTMPFILE
   $CAT $MYTMPFILE | $GAWK -v YELLOWFILE=$YELLOWLINE_FILE -v REDFILE=$REDLINE_FILE '
	BEGIN {
		FLAG=0
		hot_spares=0
		REDLINE=""
		YELLOWLINE=""
		spacer="   "
		print "<table width=\"100%\" border=0><tbody><tr><th align=center><font size=+1><b>Disk Status</b></font></th></tr>"
		print "<table width=\"100%\" border=\"1\">\n<tbody><tr><th align=\"center\">Enclosure</th><th align=\"center\">Disk 0</th><th align=\"center\">Disk 1</th><th align=\"center\">Disk 2</th><th align=\"center\">Disk 3</th><th align=\"center\">Disk 4</th><th align=\"center\">Disk 5</th><th align=\"center\">Disk 6</th><th align=\"center\">Disk 7</th><th align=\"center\">Disk 8</th><th align=\"center\">Disk 9</th></tr>"
	}
	{
		if (FLAG == 1) {
			sub ("   *","")
			sub ("State:","")
			icon=""
			status=$0
	
			FLAG=0
			if (status ~ /Hot Spare/) {
				sub ("Hot Spare ","",status)
				status="Hot Spare"
				hot_spares++
			}
			if (status ~ /Empty/) {
				icon="&clear"
			} else {
				if ((status ~ /Enabled/) || (status ~ /Ready/) || (status ~ /Hot Spare/) || (status ~ /Unbound/) || (status ~ /Binding/) || (status ~ /Expanding/)) {
					icon="&green"
				} else {
					icon="&yellow"
					YELLOWLINE=YELLOWLINE sprintf("%s%s Disk %s_%s_%s failed (%s)\n", spacer, icon, disk[1], disk[2], disk[3], status)
				}
			}
			if ((status ~ /Hot Spare/) || (status ~ /Unbound/) || (status ~ /Binding/) || (status ~ /Expanding/) || (status ~ /Removed/) || (status ~ /Rebuilding/) || (status ~ /Equalizing/))
			#if ((status !~ /Empty/) || (status !~ /Enabled/) || (status !~ /Unbound/))
			{ status="<b>"status"</b>" }
			#printf("Disk %s-%s-%s\t%s\t%s\n",disk[1],disk[2], disk[3], icon, status)
			if (disk[3] == 0) {
				printf("<tr><td align=\"left\"> Bus %s - Encl. %s </td><td align=\"left\">%s %s </td>\n", disk[1], disk[2], icon, status)
			} else { 
				if (disk[3] == 9) {
					printf("<td align=\"left\">%s %s </td></tr>\n", icon, status)
				} else { 
					printf("<td align=\"left\">%s %s </td>\n", icon, status)
				}
			}
		}
		if (($0 ~ /Bus . Enclosure . Disk ./) && (FLAG == 0)) {
			sub ("Bus ","")
			sub (" Enclosure ","-")
			sub (" Disk ","-")
			split($0,disk,"-")
			FLAG=1
		}
	}
	END {
		if (REDLINE != "" || YELLOWLINE != "") {
			if (hot_spares == 0) {
				printf("%s&red Disks are failed and no additional hot spares available\n", spacer) >> REDFILE
				printf("%s\n", YELLOWLINE) >> REDFILE
			} else {
				printf("%s&yellow Disks are failed and only %s hot spares available\n", spacer, hot_spares) >> YELLOWFILE
				printf("%s\n", YELLOWLINE) >> YELLOWFILE
			}
		}
		print "</tr></tbody></table></td></tr></tbody></table>"
		
	}' >> $MYOUTFILE

   #---[Generate information to send]---
   LINE="$($CAT $REDLINE_FILE | $SED "s/^[ 	]*$//;s/\&yellow/\&red/g")
$($CAT $YELLOWLINE_FILE | $SED "s/^[ 	]*$//")"

   #---[Check global color output]---
   YELLOWS=$(echo $LINE | $GREP \&yellow)
   [[ "$YELLOWS" != "" ]] && COLOR=yellow

   REDS=$(echo $LINE | $GREP \&red)
   if [ "$REDS" != "" ]; then
		# Check, if the SPS are recharging
		SPS_TROUBLE=$(echo $REDS | $GREP "red SPS ")
		if [ "$SPS_TROUBLE" != "" ]; then
			SPS_TESTDAY=$($SUDO $NAVICLI -h $sp_ip getcache -bbutd | $AWK '{print $4}')
			SP_CURRENTDAY=$($SUDO $NAVICLI -h $sp_ip getcontrol -day | $AWK '{print $5}')
			if [ "$SPS_TESTDAY" = "$SP_CURRENTDAY" ]; then
				SPS_TESTHOUR=$($SUDO $NAVICLI -h $sp_ip getcache -bbutt | $AWK '{print $4}' | $CUT -d":" -f1)
				SP_CURRENTHOUR=$($SUDO $NAVICLI -h $sp_ip getcontrol -time | $AWK '{print $3}' | $CUT -d":" -f1)
				(( TIME_DIFF = $SP_CURRENTHOUR - $SPS_TESTHOUR ))
				if [ $TIME_DIFF -le 1 ]; then
					COLOR=green
					SPS_CHARGING="${spacer}&green SPS is recharging - red lights on \"SP Write Cache\" and \"SPS\" are normal in this situation"
				else
					COLOR=yellow
					SPS_CHARGING="${spacer}&yellow SPS is recharging more than 1 hour - too long (since $SPS_TESTHOUR clock, now it's $SP_CURRENTHOUR)"
				fi
			else
				COLOR=yellow
				SPS_CHARGING="${spacer}&yellow SPS is recharging on $SPS_TESTDAY - Today is $SP_CURRENTDAY"
			fi
		else
      	COLOR=red
			SPS_CHARGING=""
		fi
   else
		SPS_CHARGING=""
   fi

   #---[Send page info to BBDISPLAY]---
   LINE="status $SP_FULLNAME.$TEST $COLOR $($DATE)
$HEADER
$SPS_CHARGING
$LINE
$($CAT $MYOUTFILE)"

   $BB $BBDISP "$LINE"

   $RM -f $MYOUTFILE $MYTMPFILE $YELLOWLINE_FILE $REDLINE_FILE
}

get_test_param()
{
	line=$1
	TEST_NAME=$2

	par_length=${#allowed_parameter[*]}

	if [ -n $line ]; then
		params="`$EXPR "$line" : ".*${TEST_NAME}:\(.*\)" | $AWK '{print $1}'`"
		echo $params | $TR ":" "\n" |
		while read para
		do
			para_name=${para%%=*}
			para_value=${para#*=}
			i=0
			while (($i < $par_length))
			do
				if [[ $para_name == ${allowed_parameter[i]} ]]; then
					eval "$para_name=$para_value"
					break
				fi
				((i += 1))
			done
		done
	fi
}


#
# Test-Loop
#
test_loop()
{
   $BBHOME/bin/bbhostgrep --noextras --nodown=${TEST} ${CHECK_TEST}* |
	while read linein
	do
		SP_IP=""
		SP_FULLNAME=""
		set $linein
		if [ "$#" -ne 0 ]; then
			SP_IP=$1
			SP_FULLNAME=$2
			COLOR="green"

			get_test_param "$linein" ${CHECK_TEST}

			# Test if the FibreCat is alive
			$PING -c 1 ${SP_IP}>/dev/null 2>/dev/null
			if [ "$?" -ne "0" ] ;then
				COLOR="red"
				LINE="status $SP_FULLNAME.$TEST $COLOR `date`
&red Service processor $SP_IP of ${SP_FULLNAME} could not be reached"
				$BB $BBDISP "$LINE"
			else
					get_status $SP_IP
			fi
		fi
	done
}

#
# Main
#
# now the FibreCat
test_loop

#######
# fini
#######
